#!/bin/bash

source /opt/migrate-for-gce/functions.sh

function safe_write()
{
	local src tgt
	src=$1
	tgt=$2

	sync
	mv -f ${src} ${tgt}
}

function add_default_route()
{
	local ifcfg workfile
	ifcfg=$1
	workfile=$2

	migrate_log "${ifcfg} - Setting default route"
	sed -i '/^DEFROUTE=/d' ${workfile}
	echo "DEFROUTE=yes" >> ${workfile}
}

function enforce_dhcp()
{
	local ifcfg workfile
	ifcfg=$1
	workfile=$2

	migrate_log "${ifcfg} - Reconfigure to DHCP"
	sed -i '/^BOOTPROTO=/d' ${workfile}
	echo "BOOTPROTO=dhcp" >> ${workfile}
	sed -i '/^IPADDR=/d' ${workfile}
	sed -i '/^NETMASK=/d' ${workfile}
	sed -i '/^GATEWAY=/d' ${workfile}
	sed -i '/^MTU=/d' ${workfile}

	rm -f ${ifcfg/ifcfg-/routes-}
}

function remove_hw_attachment()
{
	local ifcfg workfile
	ifcfg=$1
	workfile=$2

	migrate_log "${ifcfg} - Removing HW attachments"
	sed -i '/^HWADDR=/d' ${workfile}
	sed -i '/^MACADDR=/d' ${workfile}
	sed -i '/^MASTER=/d' ${workfile}
	sed -i '/^SLAVE=/d' ${workfile}

	sed -i '/^TYPE=/d' ${workfile}
	echo "TYPE=Ethernet" >> ${workfile}

}

function remove_device()
{
	local ifcfg workfile
	ifcfg=$1
	workfile=$2

	migrate_log "${ifcfg} - Removing DEVICE definition"
	sed -i '/^DEVICE=/d' ${workfile}
}

function set_onboot()
{
	local ifcfg workfile
	ifcfg=$1
	workfile=$2

	migrate_log "${ifcfg} - Configuring auto-start"
	sed -i '/^ONBOOT=/d' ${workfile}
	echo "ONBOOT=yes" >> ${workfile}
}

function set_user_ctl()
{
	local ifcfg workfile
	ifcfg=$1
	workfile=$2

	migrate_log "${ifcfg} - Configuring USERCTL"
	sed -i '/^USERCTL=/d' ${workfile}
	echo "USERCTL=yes" >> ${workfile}
}

function remove_static_gateway()
{
	local tmpfile

	if [ -f /etc/sysconfig/network ] ; then
		tmpfile=/etc/sysconfig/network.pre-gce
		cat /etc/sysconfig/network > ${tmpfile}
		sed -i '/^GATEWAY=/d' ${tmpfile}
		sed -i '/^GATEWAYDEV=/d' ${tmpfile}
		safe_write ${tmpfile} /etc/sysconfig/network
	fi
}

function remove_global_routes()
{
	rm -f /etc/sysconfig/routes
	rm -f /etc/sysconfig/network/routes
	rm -f /etc/sysconfig/network/routes.*
}

function remove_nic_bonding()
{
	rm -f `get_network_config_path`/ifcfg-bond*
}

function enforce_dhcp_default_route()
{
	[ -e /etc/sysconfig/network/dhcp ] && sed -i 's/DHCLIENT_SET_DEFAULT_ROUTE=.*/DHCLIENT_SET_DEFAULT_ROUTE="yes"/g' /etc/sysconfig/network/dhcp
}

function remove_static_net_config()
{
	local first tmpfile
	first=1

	for ifcfg in $(get_network_config_files | grep -v eth0); do
		tmpfile=$(dirname ${ifcfg})/migrate-for-gce.$(basename ${ifcfg})
		cat ${ifcfg} > ${tmpfile}
		enforce_dhcp ${ifcfg} ${tmpfile}
		remove_hw_attachment ${ifcfg} ${tmpfile}
		set_onboot ${ifcfg} ${tmpfile}

		if [ ${first} -eq 1 ] ; then
			add_default_route ${ifcfg} ${tmpfile}
			first=0
		fi

		safe_write ${tmpfile} ${ifcfg}
	done

	remove_static_gateway
	remove_global_routes
	remove_nic_bonding
	enforce_dhcp_default_route
}

function fix_dynamic_network_scripts()
{
	for ifcfg in $(get_network_config_files | grep -v eth0 | grep -v ens4); do
		migrate_log "Network interface configuration found in ${ifcfg}"
		tmpfile=$(dirname ${ifcfg})/migrate-for-gce.$(basename ${ifcfg})
		cat ${ifcfg} > ${tmpfile}

		remove_device ${ifcfg} ${tmpfile}
		set_user_ctl ${ifcfg} ${tmpfile}

		safe_write ${tmpfile} ${ifcfg}
	done
}

function clean_inactive_devices()
{
	local ifcfg nic

	for ifcfg in $(get_network_config_files | grep -v ifcfg-lo); do
		nic=$(basename ${ifcfg})
		if ! is_nic_attached ${nic} ; then
			migrate_log "Cleaning interface configuration ${ifcfg}"
			rm -f ${ifcfg}
		fi
	done

}

function create_default_ifcfg_scripts()
{
	write_ifcfg_script_for_iface eth0
	write_ifcfg_script_for_iface ens4
}

migrate_log "Backup network configuraiton"
mkdir -p /etc/migrate-for-gce/backup
tar zcf /etc/migrate-for-gce/backup/netconfig.tgz `get_network_config_path`

migrate_log "Running common actions"
clean_inactive_devices
remove_static_net_config
create_default_ifcfg_scripts
if is_rhel7_family || is_rhel8_family ; then
	migrate_log "Running RHEL7/8-specific actions"
	fix_dynamic_network_scripts
elif is_suse ; then
	migrate_log "Running SUSE-specific actions"
	fix_dynamic_network_scripts
fi

touch /etc/migrate-for-gce/actions/apply-net-changes
exit 0
