#!/bin/bash

migrate_state=/etc/migrate-for-gce/state
DEFAULT_GRUB_CONFIG_FILES=(/boot/grub/menu.lst /boot/grub/grub.conf /boot/grub/grub.cfg /etc/default/grub /boot/grub2/grub.cfg /boot/efi/EFI/sles/grub.cfg /boot/efi/EFI/opensuse/grub.cfg /boot/efi/EFI/redhat/grub.cfg /boot/efi/EFI/ubuntu/grub.cfg /boot/efi/EFI/centos/grub.cfg)

command_exists() {
  command -v "$1" >/dev/null 2>&1
}

# Canonicalise block device sysfs directory
#
# $1 : block device node or sysfs directory
#
function sysfs_canonical_blkdev() (
    local blkdev hexmajor hexminor major minor
    blkdev=$1

    # Convert block device node to sysfs directory
    if [ -b $blkdev ] ; then
		read hexmajor hexminor < <(stat -L -c '%t %T' $blkdev)
		major=$((0x$hexmajor))
		minor=$((0x$hexminor))
		blkdev=/sys/dev/block/$major:$minor
    fi

    # Canonicalise sysfs directory
    blkdev=$(readlink -f $blkdev)

    echo $blkdev
)

# Check for an iSCSI block device
#
# $1 : block device node or sysfs directory
#
function is_iscsi() {
    local blkdev sysfsdir slave
    blkdev=$(sysfs_canonical_blkdev $1)

    # Look for iscsi_session directory anywhere in path
    sysfsdir=$blkdev
    until [ $sysfsdir = "/sys" ] ; do
	if [ -d $sysfsdir/iscsi_session ] ; then
	    return 0
	fi
	sysfsdir=$(dirname $sysfsdir)
    done

    # Check for any slave iSCSI block devices
    if [ -d $blkdev/slaves ] ; then
	for slave in $blkdev/slaves/* ; do
	    if is_iscsi $slave ; then
		return 0
	    fi
	done
    fi

    return 1
}

# Check for an active swap device
#
# $1 : block device node or sysfs directory
#
function is_swapping() {
    local blkdev swapdev rest
    blkdev=$(sysfs_canonical_blkdev $1)

    while read swapdev rest ; do
	if [ -b $swapdev ] ; then
	    swapdev=$(sysfs_canonical_blkdev $swapdev)
	    if [ "$blkdev" = "$swapdev" ] ; then
		return 0
	    fi
	fi
    done < <(tail -n +2 /proc/swaps)

    return 1
}

function migrate_log() {
	echo "<8>migrate-for-gce: $*"  > /dev/kmsg
}

# Get network configuration files path
#
function get_network_config_path() {
	# Get proper path for the given os
	if [ -d /etc/sysconfig/network-scripts ] ; then
		echo "/etc/sysconfig/network-scripts"
	else
		echo "/etc/sysconfig/network"
	fi
}

# Get all the network interfaces (without loopback)
#
function get_network_config_files()
{
	local netscripts

	netscripts=`get_network_config_path`
	if [ -d "${netscripts}" ] ; then
		ls ${netscripts}/ifcfg-* | grep -v "ifcfg-lo"
	fi
}

# Get attached network interfaces based on device map
#
function get_attached_nics() {
	for netdev in `ls /sys/class/net/` ; do
		if [[ "${netdev}" != "lo" ]] ; then
			echo ${netdev}
		fi
	done
}

# Get attached network interfaces based on device map
# $1 : NIC name
#
function is_nic_attached() {
	local nic
	nic=$1

	for netdev in `ls /sys/class/net/` ; do
		if [[ "${netdev}" == "${nic}" ]] ; then
			return 0
		fi
	done

	return 1
}

# Get network inteface provided MAC address
# $1 : MAC address
#
function get_nic_by_hwaddr() {
	local addr
	addr=$1

	for netdev in `get_attached_nics` ; do
		mac=`cat /sys/class/net/${netdev}/address`
		if [[ "${mac}" == "${addr}" ]] ; then
			echo ${netdev}
		fi
	done
}

# Check whenever this is RHEL or CentOS
#
function is_rhel_family() {
	if [ -f "/etc/redhat-release" ]; then
		return 0
	fi

	if [ -f "/etc/centos-release" ]; then
		return 0
	fi

	return 1
}

# Check whenever this is RHEL 6 or CentOS 6
#
function is_rhel6_family() {
	if [ -f "/etc/redhat-release" ]; then
		if egrep -q "Red Hat.*release 6" /etc/redhat-release ; then
			return 0
		fi
	fi

	if [ -f "/etc/centos-release" ]; then
		if egrep -q "CentOS.*release 6" /etc/centos-release ; then
			return 0
		fi
	fi

	return 1
}

# Check whenever this is RHEL 7 or CentOS 7
#
function is_rhel7_family() {
	if [ -f "/etc/redhat-release" ]; then
		if egrep -q "Red Hat.*release 7" /etc/redhat-release ; then
			return 0
		fi
	fi

	if [ -f "/etc/centos-release" ]; then
		if egrep -q "CentOS.*release 7" /etc/centos-release ; then
			return 0
		fi
	fi

	return 1
}

# Check whenever this is RHEL 8 or CentOS 8
#
function is_rhel8_family() {
	if [ -f "/etc/redhat-release" ]; then
		if grep -Eq "Red Hat.*release 8" /etc/redhat-release ; then
			return 0
		fi
	fi

	if [ -f "/etc/centos-release" ]; then
		if grep -Eq "CentOS.*release 8" /etc/centos-release ; then
			return 0
		fi
	fi

	return 1
}

# Check whenever this is SUSE
#
function is_suse() {
  if [ -f "/etc/SuSE-release" ]; then
    return 0
  fi
  if [ -f "/etc/os-release" ] ; then
    if [ "$(. /etc/os-release && echo "${ID}")" = "suse" ] ; then
      return 0
    fi
    if [ "$(. /etc/os-release && echo "${ID_LIKE}")" = "suse" ] ; then
      return 0
    fi
  fi
  return 1
}

# Get SUSE version (e.g. 11 or 12)
#
function get_suse_version() {
	if [ -f "/etc/SuSE-release" ]; then
		grep "VERSION" /etc/SuSE-release | cut -d "=" -f 2 | tr -d "[:space:]"
	elif [ -f "/etc/os-release" ]; then
		# shellcheck disable=SC2153
		version_id="$(. /etc/os-release && echo "${VERSION_ID}")"
		# Everything before first '.'
		echo "${version_id%%.*}"
	else
		echo "0"
	fi
}

# Check SUSE service pack (e.g. 1, 2, 3, ...)
#
function get_suse_sp() {
	if [ -f "/etc/SuSE-release" ]; then
		grep PATCHLEVEL /etc/SuSE-release | cut -d "=" -f 2 | tr -d "[:space:]"
	elif [ -f "/etc/os-release" ]; then
		# shellcheck disable=SC2153
		version_id="$(. /etc/os-release && echo "${VERSION_ID}")"
		case "${version_id}" in
		*.*)
			# Everything after first '.'
			echo "${version_id#*.}"
			;;
		*)
			# If no '.' return "0"
			echo "0"
			;;
		esac
	else
		echo "0"
	fi
}

# Check whenever this is Ubuntu
#
function is_ubuntu_family() {
	if grep -q Ubuntu /etc/issue ; then
		return 0
	fi

	if grep -q Debian /etc/issue ; then
		return 0
	fi

	return 1
}

# Get Debian version (e.g. 8 or 9)
#
function get_debian_version() {
	if [ -f "/etc/issue" ]; then
		version_id="$(grep Debian /etc/issue | cut -d' ' -f3)"
		echo "${version_id%%.*}"
	elif [ -f "/etc/debian_version" ]; then
		version_id="$(cat /etc/debian_version)"
		# Everything before first '.'
		echo "${version_id%%.*}"
	else
		echo "0"
	fi
}

# Get Ubuntu version (e.g. 16 or 18)
#
function get_ubuntu_version() {
	if [ -f "/etc/issue" ]; then
		version_id="$(grep Ubuntu /etc/issue | cut -d' ' -f2)"
		echo "${version_id%%.*}"
	elif [ -f "/etc/os-release" ]; then
		version_id="$(. /etc/os-release && echo "${VERSION_ID}")"
		# Everything before first '.'
		echo "${version_id%%.*}"
	else
		echo "0"
	fi
}

# Check file for desired string, ignore commenta
# $1 : file to check
# $2 : string to find
# Returns : SUCCESS (0) if found, FAIL (1) if not found
#
function find_active_str() {
	local file nextstr

	file=$1
	shift

	if [ ! -e ${file} ]; then
		return 1
	fi

	while [[ $# -gt 0 ]] ; do
		nextstr=$1
		shift
		if [ `cat ${file} | grep -v '^\s*#' | grep -c ${nextstr}` -gt 0 ]; then
			return 0
		fi
	done

	return 1
}

# Helper method to resolve WWID-based mappings that appear in RHEL6
# The mount sources appear as /dev/mapper/WWID (wwid is unique) while
# sources were already converted to friendly names like /dev/mapper/mpatha
#
function resolve_mp_dev() {
 	local blkdev wwid mpdev
	blkdev=$1

	if [[ $blkdev == "/dev/mapper/"* ]] ; then
		wwid=`echo $blkdev | sed 's|/dev/mapper/\([0-9a-fA-F]\+\).*|\1|g'`
		mpdev=`multipath -l $wwid | grep $wwid | awk '{print $1}'`
		if [ ! -z "$mpdev" ] ; then
			echo /dev/mapper/$mpdev
			return
		fi
	fi

	echo $blkdev
}

# Check if device path matches pattern of direct mapping
# $1 : block device node
# Returns : SUCCESS (0) if found, FAIL (1) if not found
# Devices that assumed to be direct:
#   /dev/sdX
#   /dev/cciss
#   /dev/mapper/mpathX
#   /dev/dm-X
#   /dev/xvdX
#   /dev/hdX
#
function is_direct_dev() {
	if [[ $1 == /dev/sd* ]]; then
		return 0
	fi

	if [[ $1 == /dev/*cciss* ]]; then
		return 0
	fi

	if [[ $1 == /dev/*mpath* ]]; then
		return 0
	fi

	if [[ $1 == /dev/dm-* ]]; then
		return 0
	fi

	if [[ $1 == /dev/xvd* ]]; then
		return 0
	fi

	if [[ $1 == /dev/hd* ]]; then
		return 0
	fi

	return 1
}

# Check whenever the device is LVM
# $1 : block device node
# Returns : SUCCESS (0) if lvm device, FAIL (1) otherwise
#
function is_lvm_dev() {
	# Check if lvm installed
	if ! which lvs &> /dev/null; then
		return 1
	fi

	# Check whenever the device is lvm device
	lvs $1 &> /dev/null
	return $?
}

# Resolve device links to real device ID
# $1 : block device node
# Returns : real block device node (can be the same)
#
function to_real_dev() {
	local tgt

	if is_lvm_dev $1; then
		echo $1
	else
		tgt=`readlink -f $1`
		if [[ -n ${tgt} ]] ; then
			echo $tgt
		else
			echo $1
		fi
	fi
}

# Resolve device or device partition to UUID
# $1 : block device node
# Returns : UUID or empty string
#
function dev_to_uuid() {
	blkid -c /dev/null -o value -s UUID $1
}

# Go through fstab file and resolve all physical devices / partitions to UUIDs
# $1 : input fstab file (read-only)
# $2 : output fstab file
# Returns : number of resolved and replaced paths
#
function resolve_fstab_entries() {
	local src tgt dev realdev updated
	src=$1
	tgt=$2
	updated=0

	rm -f $tgt
	while IFS='' read -r line || [[ -n "${line}" ]]; do

		if [ -n "${line}" -a "${line:0:1}" != "#" ]; then
			read dev rest <<< "${line}"
			realdev=`to_real_dev ${dev}`
			if is_direct_dev ${realdev} ; then
				uuid=`dev_to_uuid ${realdev}`
				if [ -n "${uuid}" ] ; then
					echo "UUID=${uuid}  ${rest}" >> $tgt
					updated=$((updated+1))
				else
					echo "${line}" >> $tgt
				fi
			else
				echo "${line}" >> $tgt
			fi
		else
			echo "${line}" >> $tgt
		fi

	done < ${src}

	sync
	return ${updated}
}

# Set active LVM filter to desired value
# $1 : LVM config input file
# $2 : LVM config input file
# $3 : filter value
# Returns : number of replaced filters (typically 0 or 1)
#
function update_lvm_filter() {
	local src tgt filter updated
	src=$1
	tgt=$2
	filter="$3"
	updated=0

	cp -f ${src} ${tgt}
	if find_active_str ${tgt} "filter"; then
		active_filter=`cat ${tgt} | grep -v '^[ \t]*#' | grep filter | sed 's/[ \t]*filter[ \t]*=[ \t]*\[\(.*\)\]/\1/g'`
		if [[ "$active_filter" != *"a/.*/"* ]] ; then
			sed -i "s|^\([ \t]*\)\(filter[ \t]*=.*\)|\1# Updated for Google Cloud by Migrate for Compute Engine\n\1#\2\n\1filter = ${filter}|g" ${tgt}
			updated=$((updated+1))
		fi
	fi

	sync
	return ${updated}
}

# Generic method to replace string in file
# $1 : file
# $2 : source
# $3 : target
#
function replace_entry() {
	local file
	file=$1

	sed -i.tmp "s|$2|$3|g" ${file}
	sync
	rm ${file}.tmp
}

# Restart network-related service after the changes
#
function apply_net_changes() {
	migrate_log "Applying network config changes..."
	[ -e /usr/sbin/wicked ] && /usr/sbin/wicked ifreload all

	rm -f /etc/migrate-for-gce/actions/apply-net-changes
}

# Helper to check that file exists and not symbolic link
# $1 : path
#
function is_real_file() {
	if [[ -f $1 ]] && [[ ! -h $1 ]] ; then
		return 0
	fi

	return 1
}

# Detect current env
# Output values:
#   vmware
#   aws
#   azure
#   gcp
#
function detect_platform() {
	local manufacturer product

	manufacturer=$(cat /sys/devices/virtual/dmi/id/sys_vendor)
	product=$(cat /sys/devices/virtual/dmi/id/product_name)
	bios=$(cat /sys/devices/virtual/dmi/id/bios_version)

	if [[ "$product" == *"VMware"* ]] ; then
		echo vmware
	elif [[ "$bios" == *"amazon"* ]] || [[ "$product" == *"domU"* ]] || [[ "$manufacturer" == *"EC2"* ]]; then
		echo aws
	elif [[ "$product" == "Virtual Machine" ]] && [[ "$manufacturer" == *"Microsoft"* ]] ; then
		echo azure
	elif [[ "$product" == "Google Compute Engine" ]] ; then
		echo gcp
	else
		echo unknown
	fi
}


# Update platform info
#
function update_platform_info() {
	local platform btime to_clean

	btime=`cat /proc/stat | grep btime | awk '{ print $2 }'`
	to_clean=`[[ -f ${migrate_state}/last_update ]] && cat ${migrate_state}/last_update`
	if [[ "${btime}" == "${to_clean}" ]] ; then
		return
	fi

	platform=$(detect_platform)
	echo -n ${platform} > ${migrate_state}/platform.${btime}
	bash -c "[[ -e /sys/firmware/ibft ]] && echo -n 1 || echo -n 0" > ${migrate_state}/ibft.${btime}
	echo -n $btime > ${migrate_state}/last_update
	sync

	if [[ ! -z "${to_clean}" ]] ; then
		rm -f ${migrate_state}/platform.${to_clean}
		rm -f ${migrate_state}/ibft.${to_clean}
	fi
}


# Get last platform
# $1 - key to look for (e.g. platform / ibft)
# Output values:
#   vmware
#   aws
#   azure
#   gcp
#   "" (empty)
#
function get_last() {
	local key btime last_update last_entry

	key=$1
	btime=`cat /proc/stat | grep btime | awk '{ print $2 }'`
	last_update=`[[ -f ${migrate_state}/last_update ]] && cat ${migrate_state}/last_update`

	if [[ -z "${last_update}" ]] ; then
		return
	fi

	if [[ "${last_update}" == "${btime}" ]] ; then
		return
	fi

	[[ -f ${migrate_state}/${key}.${last_update} ]] && cat ${migrate_state}/${key}.${last_update}
}


# Make sure our scripts are triggered by wicked service
# $1 - script to plant
#
function install_wicked_script()
{
	local script ifcfg
	script=$1

	mkdir -p /etc/wicked/scripts/migrate-for-gce
	cp ${script} /etc/wicked/scripts/migrate-for-gce/
	for ifcfg in $(get_network_config_files | grep -v lo) ; do
		if ! grep -q "wicked:migrate-for-gce" ${ifcfg} ; then
			echo 'POST_UP_SCRIPT="wicked:migrate-for-gce"' >> ${ifcfg}
		fi
	done
}

# Install simple systemd service
# $1 - service definition
# $2 - target
#
function install_systemd_service()
{
	local svc_conf svc_name target
	svc_conf=$1
	svc_name=$(basename ${svc_conf})
	target=$2

	cp -f ${svc_conf} /lib/systemd/system/${svc_name}
	ln -sf /lib/systemd/system/${svc_name} /etc/systemd/system/${target}.wants/${svc_name}
	systemctl daemon-reload
}

get_metadata() {
  key=$1
  header="Metadata-flavor: Google"
  domain="metadata.google.internal"
  prefix="computeMetadata/v1"
  url="http://${domain}/${prefix}/${key}"

  retries=$2
  if [ -z "${retries}" ] ; then
    retries=2
  fi

  while true ; do

    if meta=$(curl -s -f -H "${header}" "${url}") ; then
      echo "${meta}"
      return 0
    fi

    retries=$((retries-1))

    if [ ${retries} -eq 0 ]; then
      break
    fi

    sleep 1
  done
  return 1
}

get_instance_metadata() {
  get_metadata "instance/$1"
}

get_gcp_license_type() {
  get_instance_metadata "attributes/license_type"
}

function write_ifcfg_script_for_iface() {
	NEW_IF=$1 # device name
	NEW_IF_SCRIPT="$(get_network_config_path)/ifcfg-$NEW_IF"
	if [ ! -f $NEW_IF_SCRIPT ]; then
		migrate_log "Creating configuration script for $NEW_IF at $NEW_IF_SCRIPT"
		echo "DEVICE=\"$NEW_IF\"" >> $NEW_IF_SCRIPT
		echo "NAME=\"$NEW_IF\"" >> $NEW_IF_SCRIPT
		echo "TYPE=\"Ethernet\"" >> $NEW_IF_SCRIPT
		echo "BOOTPROTO=\"dhcp\"" >> $NEW_IF_SCRIPT
		echo "ONBOOT=\"yes\"" >> $NEW_IF_SCRIPT
		echo "STARTMODE=\"onboot\"" >> $NEW_IF_SCRIPT
		echo "DEFROUTE=\"yes\"" >> $NEW_IF_SCRIPT

	fi
}

function get_ifcfg_scripts_archive_dir() {
	echo "$(get_network_config_path)/ifcfg_pre_gce"
}

function archive_source_ifcfg_scripts() {
	local archive_dir
	archive_dir="$(get_ifcfg_scripts_archive_dir)"
	if [ -d ${archive_dir} ]; then
		return
	fi
	mkdir -p ${archive_dir}

	for ifcfg in $(get_network_config_files); do
		migrate_log "Archiving ${ifcfg}"
		mv -f "${ifcfg}" "${archive_dir}/."
	done
}

function restore_source_ifcfg_scripts() {
	local archive_dir
	archive_dir="$(get_ifcfg_scripts_archive_dir)"
	if [ ! -d ${archive_dir} ]; then
		return
	fi

	local netscripts
	netscripts=`get_network_config_path`
	for ifcfg in `ls ${archive_dir}/ifcfg-*`; do
		migrate_log "Restoring ${ifcfg}"
		mv -f "${ifcfg}" "${netscripts}/."
	done

	rm -rf "${archive_dir}"
}
