#!/bin/bash

source /opt/migrate-for-gce/functions.sh

LOG=/etc/migrate-for-gce/gcp_agent_env.log

if is_rhel8_family; then
	NTPCONF=/etc/chrony.conf
else
	NTPCONF=/etc/ntp.conf
fi
date >> $LOG

log() {
	echo "$*" 2>&1 | tee -a $LOG
}

run_with_log() {
	log "> $*"
	"$@" 2>&1 | tee -a $LOG
}

configure_ntp() {
	cp -f ${NTPCONF} /etc/migrate-for-gce/backup/ntp.conf.pre-gcp
	sed -i 's/^pool /#pool /g' ${NTPCONF}
	if ! grep -q metadata.google.internal ${NTPCONF} ; then
		sed -i '0,/^server /s/^server /server metadata.google.internal\n&/' ${NTPCONF}
	fi
	if ! grep -q metadata.google.internal ${NTPCONF} ; then
		echo "server metadata.google.internal" >> ${NTPCONF}
	fi
	if is_rhel8_family; then
		enable_and_run_service chronyd
	else
		enable_and_run_service ntp
		enable_and_run_service ntpd
	fi
}

yum_add_google_repo() {
	local dist
	dist=$1

	# add EPEL repos
	if [ "${dist}" -gt 6 ] ; then
		run_with_log rpm --import http://dl.fedoraproject.org/pub/epel/RPM-GPG-KEY-EPEL-"${dist}"
		cat > /etc/yum.repos.d/epel.repo <<EOF
[epel]
name=Extra Packages for Enterprise Linux ${dist} - \$basearch
#baseurl=http://download.fedoraproject.org/pub/epel/${dist}/\$basearch
metalink=http://mirrors.fedoraproject.org/metalink?repo=epel-${dist}&arch=\$basearch
failovermethod=priority
enabled=1
gpgcheck=1
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-EPEL-${dist}

[epel-debuginfo]
name=Extra Packages for Enterprise Linux ${dist} - \$basearch - Debug
#baseurl=http://download.fedoraproject.org/pub/epel/${dist}/\$basearch/debug
metalink=http://mirrors.fedoraproject.org/metalink?repo=epel-debug-${dist}&arch=\$basearch
failovermethod=priority
enabled=0
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-EPEL-${dist}
gpgcheck=1

[epel-source]
name=Extra Packages for Enterprise Linux ${dist} - \$basearch - Source
#baseurl=http://download.fedoraproject.org/pub/epel/${dist}/SRPMS
metalink=http://mirrors.fedoraproject.org/metalink?repo=epel-source-${dist}&arch=\$basearch
failovermethod=priority
enabled=0
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-EPEL-${dist}
gpgcheck=1
EOF
	fi

	# add GCP repos
	cat > /etc/yum.repos.d/google-cloud.repo <<EOF
[google-compute-engine]
name=Google Compute Engine
baseurl=https://packages.cloud.google.com/yum/repos/google-compute-engine-el${dist}-x86_64-stable
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg
       https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
EOF
}

apt_add_google_repo_debian() {
	codename=$1

	cat > /etc/apt/sources.list.d/google-cloud.list <<EOF
deb http://packages.cloud.google.com/apt google-cloud-packages-archive-keyring-$codename main
EOF

}

get_zone() {
	zone=$( curl -H "Metadata-Flavor: Google" \
					"http://metadata.google.internal/computeMetadata/v1/instance/zone" \
					2> /dev/null \
					| sed 's:.*/::' )
	echo "${zone}"
}

get_region_from_zone() {
	zone=$1
	echo "${zone%-*}"
}

get_region() {
	get_region_from_zone "$(get_zone)"
}

apt_add_google_repo_ubuntu() {
	codename=$1

	repos_file="/etc/apt/sources.list"
	repos_file_backup="/etc/migrate-for-gce/backup/sources.list.pre-gce"

	mv "${repos_file}" "${repos_file_backup}"

	zone=$(get_zone)
	region=$(get_region_from_zone "${zone}")

	if [[ $( curl -s -o /dev/null -w "%{http_code}" --connect-timeout 1 "http://$region.gce.archive.ubuntu.com/ubuntu/" ) == 200 ]] ; then
		mirror="http://$region.gce.archive.ubuntu.com/ubuntu/"
	elif [[ $( curl -s -o /dev/null -w "%{http_code}" --connect-timeout 1 "http://$zone.gce.clouds.archive.ubuntu.com/ubuntu/" ) == 200 ]] ; then
		mirror="http://$zone.gce.clouds.archive.ubuntu.com/ubuntu/"
	elif [[ $( curl -s -o /dev/null -w "%{http_code}" --connect-timeout 1 "http://gce.clouds.archive.ubuntu.com/ubuntu/" ) == 200 ]] ; then
		mirror="http://gce.clouds.archive.ubuntu.com/ubuntu/"
	else
		mirror="http://archive.ubuntu.com/ubuntu"
	fi
	security="http://security.ubuntu.com/ubuntu"

	cat > "${repos_file}" << EOF
deb $mirror $codename main restricted
deb $mirror $codename-updates main restricted
deb $mirror $codename universe
deb $mirror $codename-updates universe
deb $mirror $codename multiverse
deb $mirror $codename-updates multiverse
deb $mirror $codename-backports main restricted universe multiverse
deb $security $codename-security main restricted
deb $security $codename-security universe
deb $security $codename-security multiverse
EOF
}

get_dist_id() {
  if command -v lsb_release >/dev/null 2>&1 ; then
    lsb_release -i -s | tr '[:upper:]' '[:lower:]'
  elif [ -f /etc/os-release ] ; then
    (. /etc/os-release && echo "${ID}" | tr '[:upper:]' '[:lower:]')
  fi
}

get_dist_codename() {
  if command -v lsb_release >/dev/null 2>&1 ; then
    lsb_release -c -s | tr '[:upper:]' '[:lower:]'
  elif [ -f /etc/os-release ] ; then
    (. /etc/os-release && echo "${VERSION_CODENAME}" | tr '[:upper:]' '[:lower:]')
  fi
}

apt_add_google_repo() {

	# https://cloud.google.com/vpc/docs/configure-private-google-access
	curl https://packages.cloud.google.com/apt/doc/apt-key.gpg 2>/dev/null | apt-key add -

	# only relevant for google-cloud-sdk
	codename=$(get_dist_codename)

	cloud_sdk_repo="cloud-sdk-${codename}"
	echo "deb http://packages.cloud.google.com/apt $cloud_sdk_repo main" \
		| tee -a /etc/apt/sources.list.d/google-cloud-sdk.list

	# only relevant for google-compute-engine
	compute_engine_repo="google-compute-engine-${codename}-stable"
	echo "deb http://packages.cloud.google.com/apt $compute_engine_repo main" \
		| tee -a /etc/apt/sources.list.d/google-compute-engine.list

	os_name=$(get_dist_id)
	if [[ "${os_name}" == "ubuntu" ]] ; then

		apt_add_google_repo_ubuntu "${codename}"

	elif [[ "${os_name}" == "debian" ]] ; then

		apt_add_google_repo_debian "${codename}"

	fi
}

configure_suse_connect() {
	# Following instructions on how to configure SMT servers from here:
	# https://www.suse.com/c/create-sles-demand-images-gce/
	license_type=$1
	suse_connect_file="/etc/SUSEConnect"
	suse_connect_file_backup="/etc/SUSEConnect.pre-gce"
	hosts_file="/etc/hosts"
	crt_url="http://smt-gce.susecloud.net/smt.crt"

	if [ "${license_type}" != "gcp" ]; then

		log "Skipping configuration of SUSEConnect since instance uses" \
			"${license_type} license."
		# Check if SLES is cloud licensed.
		# i.e. /etc/SUSEConnect contains:
		# "url: https://smt-azure.susecloud.net" or
		# "url: https://smt-aws.susecloud.net"
		regex="^url: *https://smt-\K[^\.]*(?=.susecloud.net)"
		if cloud="$(grep -Po "${regex}" "${suse_connect_file}")" ; then
			if [ "${cloud}" = "gce" ] ; then
			  # Instance uses "byol" license but already configured for gce
				return 0
			fi
			log "error: License type ${license_type} incompatible" \
				"with cloud configured SLES repositories from ${cloud}!"
			return 1
		fi
		return 0
	fi

	log "Configuring SUSEConnect"

	# Remove conflicting packages from other cloud providers
	run_with_log zypper -n remove regionServiceClientConfigEC2
	run_with_log zypper -n remove regionServiceClientConfigSAPEC2
	run_with_log zypper -n remove regionServiceClientConfigAzure
	run_with_log zypper -n remove regionServiceClientConfigSAPAzure

	# Suse Update infrastructure
	#https://www.suse.com/c/suse-update-infrastructure-access-through-the-data-center/
	suse_update_infra

	# Unregister instance from previous cloud
	# Do not use -d since we don't actually need to contact the previous server
	# (and can't if it's not available for us in gce)
	run_with_log SUSEConnect --cleanup
	# Remove cache
	rm -rf /etc/zypp/{repos.d,credentials.d,services.d}/*
	# Finally, register in GCE and update certificate
	run_with_log registercloudguest
	run_with_log update-ca-certificates

	return 0
}

suse_update_infra() {
	local ver
	ver=$(get_suse_version)
	rpmdir=$(mktemp -d)
	#Download link as provided in Suse documentation: 
	#https://www.suse.com/c/suse-update-infrastructure-access-through-the-data-center/
	run_with_log wget --no-check-certificate https://104.196.61.109/late_instance_offline_update_gce_SLE${ver}.tar.gz -P $rpmdir
	run_with_log tar -xf $rpmdir/late_instance_offline_update_gce_SLE${ver}.tar.gz -C $rpmdir
	run_with_log zypper --no-refresh --no-remote --non-interactive in $rpmdir/x86_64/*.rpm
}

suse_add_google_repos() {
	local ver sp license_type
	ver=$(get_suse_version)
	sp=$(get_suse_sp)
	license_type="$(get_gcp_license_type)"
	run_with_log echo Detected SUSE"${ver}" SP"${sp}"
	run_with_log echo License type: "${license_type}"

	if [[ "${ver}" == "11" ]] ; then
		run_with_log zypper modifyrepo -e nu_novell_com:SLE11-Public-Cloud-Module
	else
		configure_suse_connect "${license_type}"
		if [ "${ver}" = "15" ] && [ "${sp}" != "0" ] ; then
			public_cloud_module="sle-module-public-cloud/${ver}.${sp}/x86_64"
		else
			public_cloud_module="sle-module-public-cloud/${ver}/x86_64"
		fi
		run_with_log SUSEConnect -p "${public_cloud_module}"
	fi
	run_with_log zypper -n refs
	run_with_log zypper -n refresh

	if [ "${license_type}" == "gcp" ]; then
			suse_activate_gcp_license "${ver}"
	fi
}

suse_activate_gcp_license() {
	local ver
	ver="$1"

	run_with_log zypper -n install \
		cloud-regionsrv-client \
		cloud-regionsrv-client-plugin-gce \
		regionServiceClientConfigGCE \
		regionServiceCertsGCE

	if [[ "${ver}" == "11" ]] ; then
		/usr/sbin/registercloudguest
	elif [[ "${ver}" == "12" || "${ver}" == "15" ]] ; then
		enable_and_run_service google-clock-skew-daemon.service
		enable_and_run_service google-instance-setup.service
		enable_and_run_service google-ip-forwarding-daemon.service
		enable_and_run_service google-shutdown-scripts.service
		enable_and_run_service google-startup-scripts.service
		enable_and_run_service guestregister.service
		enable_and_run_service haveged.service
	fi

	run_with_log zypper -n refs
	run_with_log zypper -n refresh
}


rpm_remove() {
	local agent
	agent=$1

	run_with_log rpm -e "$(rpm -qa | grep "${agent}")"
}

enable_and_run_service() {
	local svc
	svc=$1

	if command -v systemctl >/dev/null ; then
		run_with_log systemctl enable "${svc}"
		run_with_log systemctl restart "${svc}"
	elif command -v update-rc.d >/dev/null ; then
		run_with_log update-rc.d "${svc}" enable
		run_with_log service "${svc}" restart
	elif command -v chkconfig >/dev/null ; then
		run_with_log chkconfig "${svc}" on
		run_with_log service "${svc}" restart
	fi
}

# Adding in GCE RHUI package
install_gce_rhui () {
	local license_type dist
	license_type=$(get_gcp_license_type)
	if is_rhel6_family ; then
		dist=6
	elif is_rhel7_family ; then
		dist=7
	elif is_rhel8_family ; then
		dist=8
	fi
	run_with_log echo Detected RHEL"${dist}"
	run_with_log echo License type: "${license_type}"

	yum_add_google_repo ${dist}

	if [ "${license_type}" == "gcp" ]; then
		if is_rhel8_family; then
			run_with_log dnf remove --cacheonly -y -- *rhui*
			run_with_log rm -rf /var/cache/dnf
			run_with_log dnf install -y google-rhui-client-rhel${dist}
		else
			run_with_log yum remove --cacheonly -y -- *rhui*
			run_with_log rm -rf /var/cache/yum
			run_with_log yum install -y google-rhui-client-rhel${dist}
		fi
	fi
}

# Check if the debian/ubuntu version support non python google agent
#
is_new_agent_support_for_ubuntu_family() {
	os_name=$(get_dist_id)
	if [ "${os_name}" = "ubuntu" ] && [ "$(get_ubuntu_version)" -ge 16 ] ; then
		return 0
	fi

	if [ "${os_name}" = "debian" ] && [ "$(get_debian_version)" -ge 9 ] ; then
		return 0
	fi
	return 1

}

# Check if the OS version support OS patch management
#
is_os_patch_management_support() {
	if [ "$(get_ubuntu_version)" -ge 16 ] ; then
		return 0
	fi

	if [ "$(get_debian_version)" -ge 9 ] ; then
		return 0
	fi

	if [ "$(get_suse_version)" -ge 12 ] ; then
		return 0
	fi

	return 1

}

# Add relevant repository for OS patch install
# https://cloud.google.com/compute/docs/manage-os#slesopensuse
#
add_compute_engine_osconfig_repository() {
	ver=$(get_suse_version)
	if [ "${ver}" -ge 12 ] ; then
		cat > /etc/zypp/repos.d/google-compute-engine.repo <<EOM
[google-compute-engine]
name=Google Compute Engine
baseurl=https://packages.cloud.google.com/yum/repos/google-compute-engine-sles${ver}-stable
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg
  https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
EOM
		rpm --import https://packages.cloud.google.com/yum/doc/yum-key.gpg --import https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
	fi
}

# Log network information before continuing
run_with_log ip addr

# 2.1. NTP configuration points to the NTP server, “metadata.google.internal”.
# 3.1. Compute Engine Linux Guest Environment is installed from packages or provided repositories.

if is_ubuntu_family ; then
	run_with_log dpkg -r amazon-ssm-agent
	run_with_log dpkg -r awscli

	export DEBIAN_FRONTEND=noninteractive

	apt_add_google_repo

	run_with_log apt-get update

	run_with_log apt-get install -y ntp
	configure_ntp

	run_with_log apt-get install -y google-cloud-packages-archive-keyring
	run_with_log apt-get update

	run_with_log apt-get install -y google-cloud-sdk

	if is_new_agent_support_for_ubuntu_family ; then
		run_with_log apt-get install -y google-compute-engine
	else
		run_with_log apt-get install -y python-google-compute-engine
		run_with_log apt-get install -y python3-google-compute-engine
	fi
	run_with_log apt-get install -y google-compute-engine-oslogin

	if is_os_patch_management_support ; then
		run_with_log apt update
		run_with_log apt-get install -y google-osconfig-agent
	fi
else
	rpm_remove amazon-ssm-agent
	rpm_remove awscli
	rpm_remove aws-cli
	rpm_remove rh-amazon-rhui-client

	if is_rhel_family ; then
		install_gce_rhui

		if is_rhel8_family; then
			run_with_log dnf install -y chrony
		else
			run_with_log yum install -y ntp
		fi
		configure_ntp

		# update python first - workaround for HA-6002
		run_with_log yum install -y python
		if is_rhel8_family; then
			run_with_log dnf install -y google-compute-engine python3-google-compute-engine
			run_with_log dnf install -y google-osconfig-agent
		else
			run_with_log yum install -y google-compute-engine python-google-compute-engine
			run_with_log yum install -y google-osconfig-agent
		fi
	elif is_suse ; then
		suse_add_google_repos

		# Wait for zypper lock if unavailable
		export ZYPP_LOCK_TIMEOUT=120

		run_with_log zypper -n install ntp
		configure_ntp

		run_with_log zypper -n install google-compute-engine-init
		run_with_log systemctl enable /usr/lib/systemd/system/google-*
		run_with_log systemctl start google-*
		run_with_log zypper -n install google-cloud-sdk
		run_with_log zypper -n install python-google-apputils
		if is_os_patch_management_support ; then
			add_compute_engine_osconfig_repository
			run_with_log zypper -n --gpg-auto-import-keys install google-osconfig-agent
		fi
	fi
fi

touch /etc/migrate-for-gce/actions/bypass/gcp-agent-env
exit 0
